import argparse

from crawlab_ai.cli.codegen import setup_codegen_parser
from crawlab_ai.cli.config import setup_config_parser
from crawlab_ai.cli.crawl import setup_crawl_parser

parser = argparse.ArgumentParser(description="Web scraping tool")
subparsers = parser.add_subparsers(dest="command")

setup_crawl_parser(subparsers)
setup_codegen_parser(subparsers)
setup_config_parser(subparsers)


def main():
    args = parser.parse_args()

    if hasattr(args, "func"):
        args.func(args)
    else:
        parser.print_help()


if __name__ == "__main__":
    main()



import os


def get_api_endpoint():
    if os.getenv("CRAWLAB_AI_API_ENDPOINT"):
        return os.getenv("CRAWLAB_AI_API_ENDPOINT")

    # Try to get the API endpoint from the file
    config_file_path = os.path.join(
        os.path.expanduser("~"), ".crawlab", "api_endpoint.json"
    )
    if os.path.exists(config_file_path):
        with open(config_file_path, "r") as f:
            return f.read()

    return "https://crawlab-ai.azurewebsites.net"


def set_api_endpoint(endpoint: str):
    config_file_path = os.path.join(
        os.path.expanduser("~"), ".crawlab", "api_endpoint.json"
    )
    with open(config_file_path, "w") as f:
        f.write(endpoint)



__all__ = [
    "read_list",
    "read_article",
    "ScrapyListSpider",
]

from crawlab_ai.scrapy.list_spider import ScrapyListSpider
from crawlab_ai.spider.article_spider import read_article
from crawlab_ai.spider.list_spider import read_list



import requests
from crawlab_ai.spider.article_spider import extract_article_rules
from crawlab_ai.utils.auth import get_auth_headers
from crawlab_ai.utils.env import get_api_endpoint


def get_code_article(url: str):
    rules = extract_article_rules(url)
    res = requests.post(
        url=get_api_endpoint() + "/code/article",
        headers=get_auth_headers(),
        json={"url": url, "rules": rules},
    )
    res.raise_for_status()
    data = res.json()
    return data["source_code"]


if __name__ == "__main__":
    print(get_code_article("https://www.36kr.com/p/2601845967059847"))



from crawlab_ai.code.article import get_code_article
from crawlab_ai.code.list import get_code_list
from crawlab_ai.utils.auth import get_token


def setup_codegen_parser(subparsers):
    codegen_parser = subparsers.add_parser(
        "codegen", help="Generate crawler code for a webpage"
    )
    codegen_parser.add_argument("url", help="URL to generate code for")
    codegen_parser.add_argument(
        "-t",
        "--type",
        help="Type of the webpage to generate code for",
        default="list",
        choices=["article", "list"],
    )
    codegen_parser.add_argument("-o", "--output", help="Output file path")
    codegen_parser.set_defaults(func=codegen)


def codegen(args):
    get_token()
    if args.type == "list":
        codegen_list(args)
    elif args.type == "article":
        codegen_article(args)


def codegen_list(args):
    code = get_code_list(args.url)
    if args.output:
        with open(args.output, "w") as f:
            f.write(code)
    else:
        print(code)


def codegen_article(args):
    code = get_code_article(args.url)
    if args.output:
        with open(args.output, "w") as f:
            f.write(code)
    else:
        print(code)



import os
from unittest.mock import patch, mock_open
from crawlab_ai.utils import auth


def test_set_token_creates_directory_and_file():
    with patch("os.path.exists", return_value=False), patch("os.makedirs"), patch(
        "builtins.open", new_callable=mock_open
    ):
        auth.set_token("test_token")
        os.makedirs.assert_called_once_with(auth.ROOT_DIRECTORY)
        open.assert_called_once_with(f"{auth.ROOT_DIRECTORY}/token", "w")


def test_set_token_writes_token_to_file():
    with patch("os.path.exists", return_value=True), patch(
        "builtins.open", new_callable=mock_open
    ) as new_mock_open:
        auth.set_token("test_token")
        new_mock_open().write.assert_called_once_with("test_token")


def test_get_token_returns_env_var_token():
    with patch.dict(os.environ, {"CRAWLAB_TOKEN": "env_token"}):
        assert auth.get_token() == "env_token"


def test_get_token_returns_file_token():
    with patch("os.path.exists", return_value=True), patch(
        "builtins.open", new_callable=mock_open, read_data="file_token"
    ):
        assert auth.get_token() == "file_token"


def test_get_token_prompts_for_token():
    with patch("os.path.exists", return_value=False), patch(
        "builtins.input", return_value="input_token"
    ), patch("crawlab_ai.utils.auth.set_token") as mock_set_token:
        assert auth.get_token() == "input_token"
        mock_set_token.assert_called_once_with("input_token")



from abc import abstractmethod

import requests


class BaseSpider(object):
    def __init__(self, url: str, get_html=None):
        self.url = url
        self.get_html = get_html or self._get_html

    @staticmethod
    def _get_html(url):
        res = requests.get(url)
        return res.text

    @abstractmethod
    def crawl(self): ...

    @abstractmethod
    def fetch_rules(self): ...



import logging

# 创建logger
logger = logging.getLogger("crawlab_ai")
logger.setLevel(logging.INFO)

# 创建handler，用于将日志输出到控制台
console_handler = logging.StreamHandler()
console_handler.setLevel(logging.INFO)

# 定义handler的输出格式
formatter = logging.Formatter("%(asctime)s [%(name)s] %(levelname)s: %(message)s")
console_handler.setFormatter(formatter)

# 给logger添加handler
logger.addHandler(console_handler)
